#include "Engine/Engine.h"
#include "Log.h"
#include "Framework/Object.h"
#include "Engine/GarbageCollector.h"

bool FluxEngine::GIsRunning = false;
FluxEngine* FluxEngine::GInstance = nullptr;

FluxEngine* FluxEngine::CreateInstance()
{
   if (GInstance != nullptr)
    {
        FluxEngine* Engine = GetInstance();

        if (Engine)
        {
            Engine->GetLogger()->LogTrace("Engine already running!");
            return nullptr;
        }
    }

    FluxEngine* Engine = new FluxEngine();
    if (Engine == nullptr)
    {
        GetLogger()->LogTrace("Failed to create engine instance!");
        return nullptr;
    }

    Engine->GetLogger()->LogTrace("Engine instance created!");
    Engine->Init();
    Engine->Start();
    return Engine;
}

GarbageCollector& FluxEngine::GetGarbageCollector()
{
    return _GarbageCollector;
}

void FluxEngine::Init()
{
    
}

FluxEngine::FluxEngine() : _Logger(*new Logger()), _GarbageCollector(*new GarbageCollector(this))
{
    if (GIsRunning)
    {
        GetLogger()->LogTrace("Engine already running!");
        bErrorState = true;
        return;
    }

    if (GInstance != nullptr)
    {
        FluxEngine* Engine = GetInstance();

        if (Engine)
        {
            Engine->GetLogger()->LogTrace("Engine already running!");
        }
        return;
    }

    GInstance = this;
}

FluxEngine::~FluxEngine()
{
    GetLogger()->LogTrace("Engine shutting down!");
}

int32_t FluxEngine::Start()
{
    if (bErrorState)
    {
        GetLogger()->LogTrace("Engine failed to start!");
        return -1;
    }

    GetLogger()->LogTrace("Engine started!");
    GIsRunning = true;
    EngineLoop();

    return bErrorState ? -1 : 0;
}

Logger* FluxEngine::GetLogger()
{
    if (GInstance != nullptr)
    {
        FluxEngine* Engine = GetInstance();
        if (Engine)
        {
            return &Engine->_Logger;
        }
    }

    static Logger DefaultLogger;
    return &DefaultLogger;
}

void FluxEngine::EngineLoop()
{
    while (IsRunning())
    {
        //GetLogger()->LogTrace("Loop running!");
        // Main engine loop
        // Update, render, etc.
    }
}
