#pragma once

#include "Core.h"
#include <memory>

#include "Framework/BaseObject.h"

class FluxEngine;

class FLUX_API Object : public BaseObject, public std::enable_shared_from_this<Object>
{
public:
    template<typename T, typename... Args>
    static std::shared_ptr<T> NewObject(Args&&... args)
    {
        static_assert(std::is_base_of<Object,T>::value, "T must inherit from Object");

        std::shared_ptr<T> ptr = std::make_shared<T>(std::forward<Args>(args)...);
        ptr->Init();
        return ptr;
    }

    Object() = default;
    virtual ~Object();

    // Copy constructor
    Object(const Object& other) = default;

    // Move constructor
    Object(Object&& other) noexcept = default;

    // Copy assignment operator
    Object& operator=(const Object& other) = default;

    // Move assignment operator
    Object& operator=(Object&& other) noexcept = default;

    virtual void Init() override;
};